 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
 #include <netinet/in.h> 
 #include <netdb.h> 
 #include <stdlib.h>

 #define DATA_BUFFER_SIZE 16384
 #define DEFAULT_ITERATIONS 100000000
 #define DATA_BUFFER "Mona Lisa was painted by Leonardo da Vinci"
 uint64_t usec_sub(struct timeval *a, struct timeval *b);

 int main(int argc, char *argv[]) 
 {
     int fd, ret_val;
     int iterations = DEFAULT_ITERATIONS;
     int buffer_size = DATA_BUFFER_SIZE;
     int counter = 0;
     long elapsed;
     struct sockaddr_in saddr;
     struct hostent *host; /* need netdb.h for this  */
     char *buffer;
     struct timeval start_time;
     struct timeval end_time;
     
     if (NULL != argv[1]){ 
        iterations = strtoul(argv[1], NULL, 0);
        /*if (ULONG_MAX == iterations){
          iterations = DEFAULT_ITERATIONS;
        }*/
     }
     if (NULL != argv[2]){
        buffer_size = strtoul(argv[2], NULL, 0);
        if (buffer_size > (DATA_BUFFER_SIZE*4) || buffer_size <= 0) {
          buffer_size = DATA_BUFFER_SIZE;
        }
     }
     else {
        buffer_size = DATA_BUFFER_SIZE;
     }
   
     buffer = malloc(buffer_size);
     while (counter < buffer_size) {
        buffer[counter] = '0' + (counter % 10);
        counter++;
     }

     printf ("Starting %i iterations of UDP connectivity\n", iterations);
     printf ("Reserving %i bytes at 0x%x\n", buffer_size, buffer);
     /* Step1: open a UDP socket */
     fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP); 
     if (fd == -1) {
         fprintf(stderr, "socket failed [%s]\n", strerror(errno));
         return -1;
     }
     printf("Created a socket with fd: %d\n", fd);

     /* Next, initialize the server address */
     saddr.sin_family = AF_INET;
     saddr.sin_port = htons(7000);
     host = gethostbyname("127.0.0.1");
     saddr.sin_addr = *((struct in_addr *)host->h_addr);
     gettimeofday(&start_time, NULL);
     printf("Started test with %i seconds %i microseconds\n", start_time.tv_sec, 
             start_time.tv_usec);
     
     while(iterations--) {
       /* Step2: send some data */
      ret_val = sendto(fd, buffer, buffer_size, 0,
          (struct sockaddr *)&saddr, sizeof(struct sockaddr_in));
      if (ret_val == -1) {
          printf("sendto() failed [%s]\n", strerror(errno));
          break;
      }
     }
     gettimeofday(&end_time, NULL);
     printf("Finished test with %i seconds %i microseconds\n", end_time.tv_sec,
             end_time.tv_usec);
     elapsed = usec_sub(&end_time, &start_time);
     printf("Elapsed time %li microseconds \n", elapsed);

     /* Last step: close the socket */
     close(fd);
     free(buffer);
     return 0;
 }


/* returns a - b in usecs */
uint64_t usec_sub(struct timeval *a, struct timeval *b)
{
  return ((uint64_t)(a->tv_sec - b->tv_sec) * 1000000ULL) +
    a->tv_usec - b->tv_usec;
}

